@@ -329,6 +329,24 @@ enum pipe_flush_flags {
  * Flags for pipe_context::memory_barrier.
  */
 #define PIPE_BARRIER_MAPPED_BUFFER     (1 << 0)
+#define PIPE_BARRIER_SHADER_BUFFER     (1 << 1)
+#define PIPE_BARRIER_QUERY_BUFFER      (1 << 2)
+#define PIPE_BARRIER_VERTEX_BUFFER     (1 << 3)
+#define PIPE_BARRIER_INDEX_BUFFER      (1 << 4)
+#define PIPE_BARRIER_CONSTANT_BUFFER   (1 << 5)
+#define PIPE_BARRIER_INDIRECT_BUFFER   (1 << 6)
+#define PIPE_BARRIER_TEXTURE           (1 << 7)
+#define PIPE_BARRIER_IMAGE             (1 << 8)
+#define PIPE_BARRIER_FRAMEBUFFER       (1 << 9)
+#define PIPE_BARRIER_STREAMOUT_BUFFER  (1 << 10)
+#define PIPE_BARRIER_GLOBAL_BUFFER     (1 << 11)
+#define PIPE_BARRIER_ALL               ((1 << 12) - 1)
+
+/**
+ * Flags for pipe_context::texture_barrier.
+ */
+#define PIPE_TEXTURE_BARRIER_SAMPLER      (1 << 0)
+#define PIPE_TEXTURE_BARRIER_FRAMEBUFFER  (1 << 1)
 
 /*
  * Resource binding flags -- state tracker must specify in advance all
@@ -446,7 +464,9 @@ enum pipe_flush_flags {
 #define PIPE_QUERY_SO_OVERFLOW_PREDICATE 8
 #define PIPE_QUERY_GPU_FINISHED          9
 #define PIPE_QUERY_PIPELINE_STATISTICS  10
-#define PIPE_QUERY_TYPES                11
+#define PIPE_QUERY_OCCLUSION_PREDICATE_CONSERVATIVE 11
+#define PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE 12
+#define PIPE_QUERY_TYPES                13
 
 /* start of driver queries,
  * see pipe_screen::get_driver_query_info */
@@ -482,6 +502,14 @@ enum pipe_flush_flags {
 
 #define PIPE_TIMEOUT_INFINITE 0xffffffffffffffffull
 
+/**
+ * pipe_image_view access flags.
+ */
+#define PIPE_IMAGE_ACCESS_READ       (1 << 0)
+#define PIPE_IMAGE_ACCESS_WRITE      (1 << 1)
+#define PIPE_IMAGE_ACCESS_READ_WRITE (PIPE_IMAGE_ACCESS_READ | \
+                                      PIPE_IMAGE_ACCESS_WRITE)
+
 /**
  * Implementation capabilities/limits which are queried through
  * pipe_screen::get_param()
