@@ -60,7 +60,16 @@ static int use_context = CONTEXT_NONE;
 
 int virgl_renderer_resource_create(struct virgl_renderer_resource_create_args *args, struct iovec *iov, uint32_t num_iovs)
 {
-   return vrend_renderer_resource_create((struct vrend_renderer_resource_create_args *)args, iov, num_iovs);
+   return vrend_renderer_resource_create((struct vrend_renderer_resource_create_args *)args, iov, num_iovs, NULL);
+}
+
+int virgl_renderer_resource_import_eglimage(struct virgl_renderer_resource_create_args *args, void *image)
+{
+#ifdef HAVE_EPOXY_EGL_H
+   return vrend_renderer_resource_create((struct vrend_renderer_resource_create_args *)args, 0, 0, image);
+#else
+   return EINVAL;
+#endif
 }
 
 void virgl_renderer_resource_unref(uint32_t res_handle)
@@ -269,7 +278,7 @@ void virgl_renderer_poll(void)
    vrend_renderer_check_fences();
 }
 
-void virgl_renderer_cleanup(void *cookie)
+void virgl_renderer_cleanup(UNUSED void *cookie)
 {
    vrend_renderer_fini();
 #ifdef HAVE_EPOXY_EGL_H
@@ -294,7 +303,7 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
    if (!cookie || !cbs)
       return -1;
 
-   if (cbs->version != 1)
+   if (cbs->version < 1 || cbs->version > VIRGL_RENDERER_CALLBACKS_VERSION)
       return -1;
 
    dev_cookie = cookie;
@@ -302,7 +311,12 @@ int virgl_renderer_init(void *cookie, int flags, struct virgl_renderer_callbacks
 
    if (flags & VIRGL_RENDERER_USE_EGL) {
 #ifdef HAVE_EPOXY_EGL_H
-      egl_info = virgl_egl_init();
+      int fd = -1;
+      if (cbs->version >= 2 && cbs->get_drm_fd) {
+         fd = cbs->get_drm_fd(cookie);
+      }
+      egl_info = virgl_egl_init(fd, flags & VIRGL_RENDERER_USE_SURFACELESS,
+                                    flags & VIRGL_RENDERER_USE_GLES);
       if (!egl_info)
          return -1;
       use_context = CONTEXT_EGL;
@@ -337,6 +351,15 @@ int virgl_renderer_get_fd_for_texture(uint32_t tex_id, int *fd)
 #endif
 }
 
+int virgl_renderer_get_fd_for_texture2(uint32_t tex_id, int *fd, int *stride, int *offset)
+{
+#ifdef HAVE_EPOXY_EGL_H
+   return virgl_egl_get_fd_for_texture2(egl_info, tex_id, fd, stride, offset);
+#else
+   return -1;
+#endif
+}
+
 void virgl_renderer_reset(void)
 {
    vrend_renderer_reset();
