@@ -49,18 +49,22 @@
 
 #include "vrend_blitter.h"
 
+#define DEST_SWIZZLE_SNIPPET_SIZE 64
+
 struct vrend_blitter_ctx {
    virgl_gl_context gl_context;
    bool initialised;
+   bool use_gles;
+   int framebuffer_srgb_enabled;
 
    GLuint vaoid;
 
    GLuint vs;
    GLuint vs_pos_only;
    GLuint fs_texfetch_col[PIPE_MAX_TEXTURE_TYPES];
-   GLuint fs_texfetch_col_emu_alpha[PIPE_MAX_TEXTURE_TYPES];
    GLuint fs_texfetch_depth[PIPE_MAX_TEXTURE_TYPES];
    GLuint fs_texfetch_depth_msaa[PIPE_MAX_TEXTURE_TYPES];
+   GLuint fs_texfetch_col_swizzle;
    GLuint fb_id;
 
    unsigned dst_width;
@@ -72,6 +76,16 @@ struct vrend_blitter_ctx {
 
 static struct vrend_blitter_ctx vrend_blit_ctx;
 
+struct vrend_blitter_point {
+    int x;
+    int y;
+};
+
+struct vrend_blitter_delta {
+    int dx;
+    int dy;
+};
+
 static bool build_and_check(GLuint id, const char *buf)
 {
    GLint param;
@@ -94,7 +108,8 @@ static bool blit_build_vs_passthrough(struct vrend_blitter_ctx *blit_ctx)
 {
    blit_ctx->vs = glCreateShader(GL_VERTEX_SHADER);
 
-   if (!build_and_check(blit_ctx->vs, VS_PASSTHROUGH)) {
+   if (!build_and_check(blit_ctx->vs,
+        blit_ctx->use_gles ? VS_PASSTHROUGH_GLES : VS_PASSTHROUGH_GL)) {
       glDeleteShader(blit_ctx->vs);
       blit_ctx->vs = 0;
       return false;
@@ -102,13 +117,64 @@ static bool blit_build_vs_passthrough(struct vrend_blitter_ctx *blit_ctx)
    return true;
 }
 
-static GLuint blit_build_frag_tex_col(struct vrend_blitter_ctx *blit_ctx, int tgsi_tex_target)
+static void create_dest_swizzle_snippet(const uint8_t swizzle[4],
+                                        char snippet[DEST_SWIZZLE_SNIPPET_SIZE])
+{
+   static const uint8_t invalid_swizzle = 0xff;
+   ssize_t si = 0;
+   uint8_t inverse[4] = {invalid_swizzle, invalid_swizzle, invalid_swizzle,
+                         invalid_swizzle};
+
+   for (int i = 0; i < 4; ++i) {
+      if (swizzle[i] > 3) continue;
+      if (inverse[swizzle[i]] == invalid_swizzle)
+         inverse[swizzle[i]] = i;
+   }
+
+   for (int i = 0; i < 4; ++i) {
+      int res = -1;
+      if (inverse[i] > 3) {
+          /* Use 0.0f for unused color values, 1.0f for an unused alpha value */
+         res = snprintf(&snippet[si], DEST_SWIZZLE_SNIPPET_SIZE - si,
+                        i < 3 ? "0.0f, " : "1.0f");
+      } else {
+         res = snprintf(&snippet[si], DEST_SWIZZLE_SNIPPET_SIZE - si,
+                        "texel.%c%s", "rgba"[inverse[i]], i < 3 ? ", " : "");
+      }
+      si += res > 0 ? res : 0;
+   }
+}
+
+static const char *vec4_type_for_tgsi_ret(enum tgsi_return_type tgsi_ret)
+{
+   switch (tgsi_ret) {
+   case TGSI_RETURN_TYPE_SINT: return "ivec4";
+   case TGSI_RETURN_TYPE_UINT: return "uvec4";
+   default: return "vec4";
+   }
+}
+
+static enum tgsi_return_type tgsi_ret_for_format(enum virgl_formats format)
+{
+   if (util_format_is_pure_uint(format))
+      return TGSI_RETURN_TYPE_UINT;
+   else if (util_format_is_pure_sint(format))
+      return TGSI_RETURN_TYPE_SINT;
+
+   return TGSI_RETURN_TYPE_UNORM;
+}
+
+static GLuint blit_build_frag_tex_col(struct vrend_blitter_ctx *blit_ctx,
+                                      int tgsi_tex_target,
+                                      enum tgsi_return_type tgsi_ret,
+                                      const uint8_t swizzle[4])
 {
    GLuint fs_id;
    char shader_buf[4096];
    int is_shad;
    const char *twm;
    const char *ext_str = "";
+   char dest_swizzle_snippet[DEST_SWIZZLE_SNIPPET_SIZE] = "texel";
    switch (tgsi_tex_target) {
    case TGSI_TEXTURE_1D:
    case TGSI_TEXTURE_BUFFER:
@@ -143,7 +209,14 @@ static GLuint blit_build_frag_tex_col(struct vrend_blitter_ctx *blit_ctx, int tg
        tgsi_tex_target == TGSI_TEXTURE_SHADOWCUBE_ARRAY)
       ext_str = "#extension GL_ARB_texture_cube_map_array : require\n";
 
-   snprintf(shader_buf, 4096, FS_TEXFETCH_COL, ext_str, vrend_shader_samplertypeconv(tgsi_tex_target, &is_shad), twm, "");
+   if (swizzle)
+      create_dest_swizzle_snippet(swizzle, dest_swizzle_snippet);
+
+   snprintf(shader_buf, 4096, blit_ctx->use_gles ? FS_TEXFETCH_COL_GLES : FS_TEXFETCH_COL_GL,
+            ext_str, vec4_type_for_tgsi_ret(tgsi_ret),
+            vrend_shader_samplerreturnconv(tgsi_ret),
+            vrend_shader_samplertypeconv(tgsi_tex_target, &is_shad), twm,
+            dest_swizzle_snippet);
 
    fs_id = glCreateShader(GL_FRAGMENT_SHADER);
 
@@ -155,48 +228,43 @@ static GLuint blit_build_frag_tex_col(struct vrend_blitter_ctx *blit_ctx, int tg
    return fs_id;
 }
 
-static GLuint blit_build_frag_tex_col_emu_alpha(struct vrend_blitter_ctx *blit_ctx, int tgsi_tex_target)
+static GLuint blit_build_frag_tex_col_msaa(struct vrend_blitter_ctx *blit_ctx,
+                                           int tgsi_tex_target,
+                                           enum tgsi_return_type tgsi_ret,
+                                           const uint8_t swizzle[4],
+                                           int nr_samples)
 {
    GLuint fs_id;
    char shader_buf[4096];
    int is_shad;
    const char *twm;
-   const char *ext_str = "";
+   const char *ivec;
+   char dest_swizzle_snippet[DEST_SWIZZLE_SNIPPET_SIZE] = "texel";
+   const char *ext_str = blit_ctx->use_gles ? "" :
+                         "#extension GL_ARB_texture_multisample : enable\n";
+
    switch (tgsi_tex_target) {
-   case TGSI_TEXTURE_1D:
-   case TGSI_TEXTURE_BUFFER:
-      twm = ".x";
-      break;
-   case TGSI_TEXTURE_1D_ARRAY:
-   case TGSI_TEXTURE_2D:
-   case TGSI_TEXTURE_RECT:
    case TGSI_TEXTURE_2D_MSAA:
-   default:
       twm = ".xy";
+      ivec = "ivec2";
       break;
-   case TGSI_TEXTURE_SHADOW1D:
-   case TGSI_TEXTURE_SHADOW2D:
-   case TGSI_TEXTURE_SHADOW1D_ARRAY:
-   case TGSI_TEXTURE_SHADOWRECT:
-   case TGSI_TEXTURE_3D:
-   case TGSI_TEXTURE_CUBE:
-   case TGSI_TEXTURE_2D_ARRAY:
    case TGSI_TEXTURE_2D_ARRAY_MSAA:
       twm = ".xyz";
+      ivec = "ivec3";
       break;
-   case TGSI_TEXTURE_SHADOWCUBE:
-   case TGSI_TEXTURE_SHADOW2D_ARRAY:
-   case TGSI_TEXTURE_SHADOWCUBE_ARRAY:
-   case TGSI_TEXTURE_CUBE_ARRAY:
-      twm = "";
-      break;
+   default:
+      return 0;
    }
 
-   if (tgsi_tex_target == TGSI_TEXTURE_CUBE_ARRAY ||
-       tgsi_tex_target == TGSI_TEXTURE_SHADOWCUBE_ARRAY)
-      ext_str = "#extension GL_ARB_texture_cube_map_array : require\n";
+   if (swizzle)
+      create_dest_swizzle_snippet(swizzle, dest_swizzle_snippet);
 
-   snprintf(shader_buf, 4096, FS_TEXFETCH_COL_ALPHA_DEST, ext_str, vrend_shader_samplertypeconv(tgsi_tex_target, &is_shad), twm, "");
+   snprintf(shader_buf, 4096,
+            blit_ctx->use_gles ? FS_TEXFETCH_COL_MSAA_GLES : FS_TEXFETCH_COL_MSAA_GL,
+            ext_str, vec4_type_for_tgsi_ret(tgsi_ret),
+            vrend_shader_samplerreturnconv(tgsi_ret),
+            vrend_shader_samplertypeconv(tgsi_tex_target, &is_shad),
+            nr_samples, ivec, twm, dest_swizzle_snippet);
 
    fs_id = glCreateShader(GL_FRAGMENT_SHADER);
 
@@ -245,7 +313,8 @@ static GLuint blit_build_frag_tex_writedepth(struct vrend_blitter_ctx *blit_ctx,
       break;
    }
 
-   snprintf(shader_buf, 4096, FS_TEXFETCH_DS, vrend_shader_samplertypeconv(tgsi_tex_target, &is_shad), twm);
+   snprintf(shader_buf, 4096, blit_ctx->use_gles ? FS_TEXFETCH_DS_GLES : FS_TEXFETCH_DS_GL,
+      vrend_shader_samplertypeconv(tgsi_tex_target, &is_shad), twm);
 
    fs_id = glCreateShader(GL_FRAGMENT_SHADER);
 
@@ -278,7 +347,8 @@ static GLuint blit_build_frag_blit_msaa_depth(struct vrend_blitter_ctx *blit_ctx
       return 0;
    }
 
-   snprintf(shader_buf, 4096, FS_TEXFETCH_DS_MSAA, vrend_shader_samplertypeconv(tgsi_tex_target, &is_shad), ivec, twm);
+   snprintf(shader_buf, 4096, blit_ctx->use_gles ? FS_TEXFETCH_DS_MSAA_GLES : FS_TEXFETCH_DS_MSAA_GL,
+      vrend_shader_samplertypeconv(tgsi_tex_target, &is_shad), ivec, twm);
 
    fs_id = glCreateShader(GL_FRAGMENT_SHADER);
 
@@ -316,37 +386,44 @@ static GLuint blit_get_frag_tex_writedepth(struct vrend_blitter_ctx *blit_ctx, i
    }
 }
 
-static GLuint blit_get_frag_tex_col(struct vrend_blitter_ctx *blit_ctx, int pipe_tex_target, unsigned nr_samples)
+static GLuint blit_get_frag_tex_col(struct vrend_blitter_ctx *blit_ctx,
+                                    int pipe_tex_target,
+                                    unsigned nr_samples,
+                                    const struct vrend_format_table *src_entry,
+                                    const struct vrend_format_table *dst_entry)
 {
    assert(pipe_tex_target < PIPE_MAX_TEXTURE_TYPES);
 
-   if (nr_samples > 1) {
-      return 0;
+   bool needs_swizzle = dst_entry->flags & VIRGL_BIND_NEED_SWIZZLE;
+
+   if (needs_swizzle || nr_samples > 1) {
+      const uint8_t *swizzle = needs_swizzle ? dst_entry->swizzle : NULL;
+      GLuint *shader = &blit_ctx->fs_texfetch_col_swizzle;
+      if (shader) {
+         glDeleteShader(*shader);
+      }
+
+      unsigned tgsi_tex = util_pipe_tex_to_tgsi_tex(pipe_tex_target, nr_samples);
+      enum tgsi_return_type tgsi_ret = tgsi_ret_for_format(src_entry->format);
+
+      if (nr_samples > 1) {
+         // Integer textures are resolved using just one sample
+         int msaa_samples = tgsi_ret == TGSI_RETURN_TYPE_UNORM ? nr_samples : 1;
+         *shader = blit_build_frag_tex_col_msaa(blit_ctx, tgsi_tex, tgsi_ret,
+                                                swizzle, msaa_samples);
+      } else {
+         *shader = blit_build_frag_tex_col(blit_ctx, tgsi_tex, tgsi_ret, swizzle);
+      }
+
+      return *shader;
    } else {
       GLuint *shader = &blit_ctx->fs_texfetch_col[pipe_tex_target];
 
       if (!*shader) {
          unsigned tgsi_tex = util_pipe_tex_to_tgsi_tex(pipe_tex_target, 0);
+         enum tgsi_return_type tgsi_ret = tgsi_ret_for_format(src_entry->format);
 
-         *shader = blit_build_frag_tex_col(blit_ctx, tgsi_tex);
-      }
-      return *shader;
-   }
-}
-
-static GLuint blit_get_frag_tex_col_emu_alpha(struct vrend_blitter_ctx *blit_ctx, int pipe_tex_target, unsigned nr_samples)
-{
-   assert(pipe_tex_target < PIPE_MAX_TEXTURE_TYPES);
-
-   if (nr_samples > 1) {
-      return 0;
-   } else {
-      GLuint *shader = &blit_ctx->fs_texfetch_col_emu_alpha[pipe_tex_target];
-
-      if (!*shader) {
-         unsigned tgsi_tex = util_pipe_tex_to_tgsi_tex(pipe_tex_target, 0);
-
-         *shader = blit_build_frag_tex_col_emu_alpha(blit_ctx, tgsi_tex);
+         *shader = blit_build_frag_tex_col(blit_ctx, tgsi_tex, tgsi_ret, NULL);
       }
       return *shader;
    }
@@ -362,10 +439,16 @@ static void vrend_renderer_init_blit_ctx(struct vrend_blitter_ctx *blit_ctx)
    }
 
    blit_ctx->initialised = true;
+   blit_ctx->use_gles = epoxy_is_desktop_gl() == 0;
    ctx_params.shared = true;
-   ctx_params.major_ver = VREND_GL_VER_MAJOR;
-   ctx_params.minor_ver = VREND_GL_VER_MINOR;
-   blit_ctx->gl_context = vrend_clicbs->create_gl_context(0, &ctx_params);
+   for (uint32_t i = 0; i < ARRAY_SIZE(gl_versions); i++) {
+      ctx_params.major_ver = gl_versions[i].major;
+      ctx_params.minor_ver = gl_versions[i].minor;
+
+      blit_ctx->gl_context = vrend_clicbs->create_gl_context(0, &ctx_params);
+      if (blit_ctx->gl_context)
+         break;
+   }
 
    vrend_clicbs->make_current(0, blit_ctx->gl_context);
    glGenVertexArrays(1, &blit_ctx->vaoid);
@@ -378,6 +461,10 @@ static void vrend_renderer_init_blit_ctx(struct vrend_blitter_ctx *blit_ctx)
       blit_ctx->vertices[i][0][3] = 1; /*v.w*/
    glBindVertexArray(blit_ctx->vaoid);
    glBindBuffer(GL_ARRAY_BUFFER, blit_ctx->vbo_id);
+
+   if (!blit_ctx->use_gles)
+      glEnable(GL_FRAMEBUFFER_SRGB);
+   blit_ctx->framebuffer_srgb_enabled = true;
 }
 
 static inline GLenum convert_mag_filter(unsigned int filter)
@@ -528,13 +615,87 @@ static void set_dsa_write_depth_keep_stencil(void)
    glDepthMask(GL_TRUE);
 }
 
+static inline GLenum to_gl_swizzle(int swizzle)
+{
+   switch (swizzle) {
+   case PIPE_SWIZZLE_RED: return GL_RED;
+   case PIPE_SWIZZLE_GREEN: return GL_GREEN;
+   case PIPE_SWIZZLE_BLUE: return GL_BLUE;
+   case PIPE_SWIZZLE_ALPHA: return GL_ALPHA;
+   case PIPE_SWIZZLE_ZERO: return GL_ZERO;
+   case PIPE_SWIZZLE_ONE: return GL_ONE;
+   default:
+      assert(0);
+      return 0;
+   }
+}
+
+/* Calculate the delta required to keep 'v' within [0, max] */
+static int calc_delta_for_bound(int v, int max)
+{
+    int delta = 0;
+
+    if (v < 0)
+        delta = -v;
+    else if (v > max)
+        delta = - (v - max);
+
+    return delta;
+}
+
+/* Calculate the deltas for the source blit region points in order to bound
+ * them within the source resource extents */
+static void calc_src_deltas_for_bounds(struct vrend_resource *src_res,
+                                       const struct pipe_blit_info *info,
+                                       struct vrend_blitter_delta *src0_delta,
+                                       struct vrend_blitter_delta *src1_delta)
+{
+   int max_x = u_minify(src_res->base.width0, info->src.level) - 1;
+   int max_y = u_minify(src_res->base.height0, info->src.level) - 1;
+
+   /* Whether the bounds for the coordinates of a point are inclusive or
+    * exclusive depends on the direction of the blit read. Adjust the max
+    * bounds accordingly, with an adjustment of 0 for inclusive, and 1 for
+    * exclusive. */
+   int src0_x_excl = info->src.box.width < 0;
+   int src0_y_excl = info->src.box.height < 0;
+
+   src0_delta->dx = calc_delta_for_bound(info->src.box.x, max_x + src0_x_excl);
+   src0_delta->dy = calc_delta_for_bound(info->src.box.y, max_y + src0_y_excl);
+
+   src1_delta->dx = calc_delta_for_bound(info->src.box.x + info->src.box.width,
+                                         max_x + !src0_x_excl);
+   src1_delta->dy = calc_delta_for_bound(info->src.box.y + info->src.box.height,
+                                         max_y + !src0_y_excl);
+}
+
+/* Calculate dst delta values to adjust the dst points for any changes in the
+ * src points */
+static void calc_dst_deltas_from_src(const struct pipe_blit_info *info,
+                                     const struct vrend_blitter_delta *src0_delta,
+                                     const struct vrend_blitter_delta *src1_delta,
+                                     struct vrend_blitter_delta *dst0_delta,
+                                     struct vrend_blitter_delta *dst1_delta)
+{
+   float scale_x = (float)info->dst.box.width / (float)info->src.box.width;
+   float scale_y = (float)info->dst.box.height / (float)info->src.box.height;
+
+   dst0_delta->dx = src0_delta->dx * scale_x;
+   dst0_delta->dy = src0_delta->dy * scale_y;
+
+   dst1_delta->dx = src1_delta->dx * scale_x;
+   dst1_delta->dy = src1_delta->dy * scale_y;
+}
+
 /* implement blitting using OpenGL. */
-void vrend_renderer_blit_gl(struct vrend_context *ctx,
+void vrend_renderer_blit_gl(UNUSED struct vrend_context *ctx,
                             struct vrend_resource *src_res,
                             struct vrend_resource *dst_res,
-                            const struct pipe_blit_info *info)
+                            const struct pipe_blit_info *info,
+                            bool has_texture_srgb_decode)
 {
    struct vrend_blitter_ctx *blit_ctx = &vrend_blit_ctx;
+   GLuint buffers;
    GLuint prog_id;
    GLuint fs_id;
    GLint lret;
@@ -544,10 +705,16 @@ void vrend_renderer_blit_gl(struct vrend_context *ctx,
    bool has_depth, has_stencil;
    bool blit_stencil, blit_depth;
    int dst_z;
+   struct vrend_blitter_delta src0_delta, src1_delta, dst0_delta, dst1_delta;
+   struct vrend_blitter_point src0, src1, dst0, dst1;
    const struct util_format_description *src_desc =
       util_format_description(src_res->base.format);
    const struct util_format_description *dst_desc =
       util_format_description(dst_res->base.format);
+   const struct vrend_format_table *src_entry =
+      vrend_get_format_table_entry(info->src.format);
+   const struct vrend_format_table *dst_entry =
+      vrend_get_format_table_entry(info->dst.format);
 
    has_depth = util_format_has_depth(src_desc) &&
       util_format_has_depth(dst_desc);
@@ -564,20 +731,33 @@ void vrend_renderer_blit_gl(struct vrend_context *ctx,
                        u_minify(dst_res->base.width0, info->dst.level),
                        u_minify(dst_res->base.height0, info->dst.level));
 
-   blitter_set_rectangle(blit_ctx, info->dst.box.x, info->dst.box.y,
-                         info->dst.box.x + info->dst.box.width,
-                         info->dst.box.y + info->dst.box.height, 0);
+   /* Calculate src and dst points taking deltas into account */
+   calc_src_deltas_for_bounds(src_res, info, &src0_delta, &src1_delta);
+   calc_dst_deltas_from_src(info, &src0_delta, &src1_delta, &dst0_delta, &dst1_delta);
 
+   src0.x = info->src.box.x + src0_delta.dx;
+   src0.y = info->src.box.y + src0_delta.dy;
+   src1.x = info->src.box.x + info->src.box.width + src1_delta.dx;
+   src1.y = info->src.box.y + info->src.box.height + src1_delta.dy;
+
+   dst0.x = info->dst.box.x + dst0_delta.dx;
+   dst0.y = info->dst.box.y + dst0_delta.dy;
+   dst1.x = info->dst.box.x + info->dst.box.width + dst1_delta.dx;
+   dst1.y = info->dst.box.y + info->dst.box.height + dst1_delta.dy;
+
+   blitter_set_rectangle(blit_ctx, dst0.x, dst0.y, dst1.x, dst1.y, 0);
 
    prog_id = glCreateProgram();
    glAttachShader(prog_id, blit_ctx->vs);
 
-   if (blit_depth || blit_stencil)
-      fs_id = blit_get_frag_tex_writedepth(blit_ctx, src_res->base.target, src_res->base.nr_samples);
-   else if (vrend_format_is_emulated_alpha(info->dst.format))
-      fs_id = blit_get_frag_tex_col_emu_alpha(blit_ctx, src_res->base.target, src_res->base.nr_samples);
-   else
-      fs_id = blit_get_frag_tex_col(blit_ctx, src_res->base.target, src_res->base.nr_samples);
+   if (blit_depth || blit_stencil) {
+      fs_id = blit_get_frag_tex_writedepth(blit_ctx, src_res->base.target,
+                                           src_res->base.nr_samples);
+   } else {
+      fs_id = blit_get_frag_tex_col(blit_ctx, src_res->base.target,
+                                    src_res->base.nr_samples,
+                                    src_entry, dst_entry);
+   }
    glAttachShader(prog_id, fs_id);
 
    glLinkProgram(prog_id);
@@ -597,21 +777,39 @@ void vrend_renderer_blit_gl(struct vrend_context *ctx,
    glBindFramebuffer(GL_FRAMEBUFFER_EXT, blit_ctx->fb_id);
    vrend_fb_bind_texture(dst_res, 0, info->dst.level, info->dst.box.z);
 
-   glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
+   buffers = GL_COLOR_ATTACHMENT0_EXT;
+   glDrawBuffers(1, &buffers);
 
    glBindTexture(src_res->target, src_res->id);
 
-   if (vrend_format_is_emulated_alpha(info->src.format))
-      glTexParameteri(src_res->target, GL_TEXTURE_SWIZZLE_R, GL_ALPHA);
+   if (src_entry->flags & VIRGL_BIND_NEED_SWIZZLE) {
+      glTexParameteri(src_res->target, GL_TEXTURE_SWIZZLE_R,
+                      to_gl_swizzle(src_entry->swizzle[0]));
+      glTexParameteri(src_res->target, GL_TEXTURE_SWIZZLE_G,
+                      to_gl_swizzle(src_entry->swizzle[1]));
+      glTexParameteri(src_res->target, GL_TEXTURE_SWIZZLE_B,
+                      to_gl_swizzle(src_entry->swizzle[2]));
+      glTexParameteri(src_res->target, GL_TEXTURE_SWIZZLE_A,
+                      to_gl_swizzle(src_entry->swizzle[3]));
+   }
 
-   glTexParameteri(src_res->target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-   glTexParameteri(src_res->target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-   glTexParameteri(src_res->target, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
+   /* Just make sure that no stale state disabled decoding */
+   if (has_texture_srgb_decode && util_format_is_srgb(src_res->base.format))
+         glTexParameteri(src_res->target, GL_TEXTURE_SRGB_DECODE_EXT, GL_DECODE_EXT);
+
+   if (src_res->base.nr_samples <= 1) {
+      glTexParameteri(src_res->target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+      glTexParameteri(src_res->target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+      glTexParameteri(src_res->target, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
+   }
 
    glTexParameteri(src_res->target, GL_TEXTURE_BASE_LEVEL, info->src.level);
    glTexParameteri(src_res->target, GL_TEXTURE_MAX_LEVEL, info->src.level);
-   glTexParameterf(src_res->target, GL_TEXTURE_MAG_FILTER, filter);
-   glTexParameterf(src_res->target, GL_TEXTURE_MIN_FILTER, filter);
+
+   if (src_res->base.nr_samples <= 1) {
+      glTexParameterf(src_res->target, GL_TEXTURE_MAG_FILTER, filter);
+      glTexParameterf(src_res->target, GL_TEXTURE_MIN_FILTER, filter);
+   }
    pos_loc = glGetAttribLocation(prog_id, "arg0");
    tc_loc = glGetAttribLocation(prog_id, "arg1");
    samp_loc = glGetUniformLocation(prog_id, "samp");
@@ -631,18 +829,32 @@ void vrend_renderer_blit_gl(struct vrend_context *ctx,
       float dst_offset = ((info->src.box.depth - 1) -
                           (info->dst.box.depth - 1) * dst2src_scale) * 0.5;
       float src_z = (dst_z + dst_offset) * dst2src_scale;
+      uint32_t layer = (dst_res->target == GL_TEXTURE_CUBE_MAP) ? info->dst.box.z : dst_z;
 
       glBindFramebuffer(GL_FRAMEBUFFER_EXT, blit_ctx->fb_id);
-      vrend_fb_bind_texture(dst_res, 0, info->dst.level, dst_z);
+      vrend_fb_bind_texture(dst_res, 0, info->dst.level, layer);
 
-      glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
+      buffers = GL_COLOR_ATTACHMENT0_EXT;
+      glDrawBuffers(1, &buffers);
       blitter_set_texcoords(blit_ctx, src_res, info->src.level,
                             info->src.box.z + src_z, 0,
-                            info->src.box.x, info->src.box.y,
-                            info->src.box.x + info->src.box.width,
-                            info->src.box.y + info->src.box.height);
+                            src0.x, src0.y, src1.x, src1.y);
 
       glBufferData(GL_ARRAY_BUFFER, sizeof(blit_ctx->vertices), blit_ctx->vertices, GL_STATIC_DRAW);
       glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
    }
+
+   glUseProgram(0);
+   glDeleteProgram(prog_id);
+   glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_STENCIL_ATTACHMENT,
+                             GL_TEXTURE_2D, 0, 0);
+   glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0,
+                             GL_TEXTURE_2D, 0, 0);
+}
+
+void vrend_blitter_fini(void)
+{
+   vrend_blit_ctx.initialised = false;
+   vrend_clicbs->destroy_gl_context(vrend_blit_ctx.gl_context);
+   memset(&vrend_blit_ctx, 0, sizeof(vrend_blit_ctx));
 }
