@@ -26,8 +26,16 @@
 
 /* shaders for blitting */
 
-#define VS_PASSTHROUGH                          \
+#define HEADER_GL                               \
+   "// Blitter\n"                               \
    "#version 130\n"                             \
+
+#define HEADER_GLES                             \
+   "// Blitter\n"                               \
+   "#version 310 es\n"                          \
+   "precision mediump float;\n"                 \
+
+#define VS_PASSTHROUGH_BODY                     \
    "in vec4 arg0;\n"                            \
    "in vec4 arg1;\n"                            \
    "out vec4 tc;\n"                             \
@@ -36,40 +44,64 @@
    "   tc = arg1;\n"                            \
    "}\n"
 
-#define FS_TEXFETCH_COL                         \
-   "#version 130\n"                             \
+#define VS_PASSTHROUGH_GL HEADER_GL VS_PASSTHROUGH_BODY
+#define VS_PASSTHROUGH_GLES HEADER_GLES VS_PASSTHROUGH_BODY
+
+
+#define FS_TEXFETCH_COL_BODY                    \
    "%s"                                         \
-   "uniform sampler%s samp;\n"                  \
+   "#define cvec4 %s\n"                         \
+   "uniform mediump %csampler%s samp;\n"        \
    "in vec4 tc;\n"                              \
+   "out cvec4 FragColor;\n"                     \
    "void main() {\n"                            \
-   "   gl_FragColor = texture(samp, tc%s)%s;\n" \
+   "   cvec4 texel = texture(samp, tc%s);\n"    \
+   "   FragColor = cvec4(%s);\n"                \
    "}\n"
 
-#define FS_TEXFETCH_COL_ALPHA_DEST              \
-   "#version 130\n"                             \
-   "%s"                                         \
-   "uniform sampler%s samp;\n"                  \
-   "in vec4 tc;\n"                              \
-   "void main() {\n"                            \
-   "   vec4 temp = texture(samp, tc%s)%s;\n"     \
-   "   gl_FragColor = temp.aaaa;\n" \
+#define FS_TEXFETCH_COL_GL HEADER_GL FS_TEXFETCH_COL_BODY
+#define FS_TEXFETCH_COL_GLES HEADER_GLES FS_TEXFETCH_COL_BODY
+
+
+#define FS_TEXFETCH_COL_MSAA_BODY                       \
+   "%s"                                                 \
+   "#define cvec4 %s\n"                                 \
+   "uniform mediump %csampler%s samp;\n"                \
+   "in vec4 tc;\n"                                      \
+   "out cvec4 FragColor;\n"                             \
+   "void main() {\n"                                    \
+   "   const int num_samples = %d;\n"                   \
+   "   cvec4 texel = cvec4(0);\n"                       \
+   "   for (int i = 0; i < num_samples; ++i) \n"        \
+   "      texel += texelFetch(samp, %s(tc%s), i);\n"    \
+   "   texel = texel / cvec4(num_samples);\n"           \
+   "   FragColor = cvec4(%s);\n"                        \
    "}\n"
 
-#define FS_TEXFETCH_DS                                  \
-   "#version 130\n"                                     \
+#define FS_TEXFETCH_COL_MSAA_GL HEADER_GL FS_TEXFETCH_COL_MSAA_BODY
+#define FS_TEXFETCH_COL_MSAA_GLES HEADER_GLES FS_TEXFETCH_COL_MSAA_BODY
+
+#define FS_TEXFETCH_DS_BODY                             \
    "uniform sampler%s samp;\n"                          \
    "in vec4 tc;\n"                                      \
    "void main() {\n"                                    \
    "   gl_FragDepth = float(texture(samp, tc%s).x);\n"  \
    "}\n"
 
-#define FS_TEXFETCH_DS_MSAA                                             \
-   "#version 130\n"                                                     \
-   "#extension GL_ARB_texture_multisample : enable\n"                   \
-   "uniform sampler%s samp;\n"                                          \
-   "in vec4 tc;\n"                                                      \
-   "void main() {\n"                                                    \
+#define FS_TEXFETCH_DS_GL HEADER_GL FS_TEXFETCH_DS_BODY
+#define FS_TEXFETCH_DS_GLES HEADER_GLES FS_TEXFETCH_DS_BODY
+
+
+#define FS_TEXFETCH_DS_MSAA_BODY                                         \
+   "#extension GL_ARB_texture_multisample : enable\n"                    \
+   "uniform sampler%s samp;\n"                                           \
+   "in vec4 tc;\n"                                                       \
+   "void main() {\n"                                                     \
    "   gl_FragDepth = float(texelFetch(samp, %s(tc%s), int(tc.z)).x);\n" \
    "}\n"
 
+#define FS_TEXFETCH_DS_MSAA_GL HEADER_GL FS_TEXFETCH_DS_MSAA_BODY
+#define FS_TEXFETCH_DS_MSAA_GLES HEADER_GLES FS_TEXFETCH_DS_MSAA_BODY
+
+
 #endif
