@@ -47,7 +47,7 @@ struct vrend_decode_ctx {
    struct vrend_decoder_state ids, *ds;
    struct vrend_context *grctx;
 };
-#define VREND_MAX_CTX 16
+#define VREND_MAX_CTX 64
 static struct vrend_decode_ctx *dec_ctx[VREND_MAX_CTX];
 
 static inline uint32_t get_buf_entry(struct vrend_decode_ctx *ctx, uint32_t offset)
@@ -66,22 +66,31 @@ static int vrend_decode_create_shader(struct vrend_decode_ctx *ctx,
                                       uint16_t length)
 {
    struct pipe_stream_output_info so_info;
-   int i, ret;
-   uint32_t shader_offset;
+   uint i;
+   int ret;
+   uint32_t shader_offset, req_local_mem = 0;
    unsigned num_tokens, num_so_outputs, offlen;
    uint8_t *shd_text;
    uint32_t type;
 
-   if (length < 5)
+   if (length < VIRGL_OBJ_SHADER_HDR_SIZE(0))
       return EINVAL;
 
    type = get_buf_entry(ctx, VIRGL_OBJ_SHADER_TYPE);
    num_tokens = get_buf_entry(ctx, VIRGL_OBJ_SHADER_NUM_TOKENS);
    offlen = get_buf_entry(ctx, VIRGL_OBJ_SHADER_OFFSET);
-   num_so_outputs = get_buf_entry(ctx, VIRGL_OBJ_SHADER_SO_NUM_OUTPUTS);
 
-   if (num_so_outputs > PIPE_MAX_SO_OUTPUTS)
-      return EINVAL;
+   if (type == PIPE_SHADER_COMPUTE) {
+      req_local_mem = get_buf_entry(ctx, VIRGL_OBJ_SHADER_SO_NUM_OUTPUTS);
+      num_so_outputs = 0;
+   } else {
+      num_so_outputs = get_buf_entry(ctx, VIRGL_OBJ_SHADER_SO_NUM_OUTPUTS);
+      if (length < VIRGL_OBJ_SHADER_HDR_SIZE(num_so_outputs))
+         return EINVAL;
+
+      if (num_so_outputs > PIPE_MAX_SO_OUTPUTS)
+         return EINVAL;
+   }
 
    shader_offset = 6;
    if (num_so_outputs) {
@@ -97,6 +106,8 @@ static int vrend_decode_create_shader(struct vrend_decode_ctx *ctx,
             so_info.output[i].num_components = (tmp >> 10) & 0x7;
             so_info.output[i].output_buffer = (tmp >> 13) & 0x7;
             so_info.output[i].dst_offset = (tmp >> 16) & 0xffff;
+            tmp = get_buf_entry(ctx, VIRGL_OBJ_SHADER_SO_OUTPUT0_SO(i));
+            so_info.output[i].stream = (tmp & 0x3);
          }
       }
       shader_offset += 4 + (2 * num_so_outputs);
@@ -104,7 +115,7 @@ static int vrend_decode_create_shader(struct vrend_decode_ctx *ctx,
      memset(&so_info, 0, sizeof(so_info));
 
    shd_text = get_buf_ptr(ctx, shader_offset);
-   ret = vrend_create_shader(ctx->grctx, handle, &so_info, (const char *)shd_text, offlen, num_tokens, type, length - shader_offset + 1);
+   ret = vrend_create_shader(ctx->grctx, handle, &so_info, req_local_mem, (const char *)shd_text, offlen, num_tokens, type, length - shader_offset + 1);
 
    return ret;
 }
@@ -129,7 +140,7 @@ static int vrend_decode_set_framebuffer_state(struct vrend_decode_ctx *ctx, int
    if (length < 2)
       return EINVAL;
 
-   uint32_t nr_cbufs = get_buf_entry(ctx, VIRGL_SET_FRAMEBUFFER_STATE_NR_CBUFS);
+   int32_t nr_cbufs = get_buf_entry(ctx, VIRGL_SET_FRAMEBUFFER_STATE_NR_CBUFS);
    uint32_t zsurf_handle = get_buf_entry(ctx, VIRGL_SET_FRAMEBUFFER_STATE_NR_ZSURF_HANDLE);
    uint32_t surf_handle[8];
    int i;
@@ -146,6 +157,27 @@ static int vrend_decode_set_framebuffer_state(struct vrend_decode_ctx *ctx, int
    return 0;
 }
 
+static int vrend_decode_set_framebuffer_state_no_attach(struct vrend_decode_ctx *ctx, int length)
+{
+   uint32_t width, height;
+   uint32_t layers, samples;
+   uint32_t tmp;
+
+   if (length != VIRGL_SET_FRAMEBUFFER_STATE_NO_ATTACH_SIZE)
+      return EINVAL;
+
+   tmp = get_buf_entry(ctx, VIRGL_SET_FRAMEBUFFER_STATE_NO_ATTACH_WIDTH_HEIGHT);
+   width = VIRGL_SET_FRAMEBUFFER_STATE_NO_ATTACH_WIDTH(tmp);
+   height = VIRGL_SET_FRAMEBUFFER_STATE_NO_ATTACH_HEIGHT(tmp);
+
+   tmp = get_buf_entry(ctx, VIRGL_SET_FRAMEBUFFER_STATE_NO_ATTACH_LAYERS_SAMPLES);
+   layers = VIRGL_SET_FRAMEBUFFER_STATE_NO_ATTACH_LAYERS(tmp);
+   samples = VIRGL_SET_FRAMEBUFFER_STATE_NO_ATTACH_SAMPLES(tmp);
+
+   vrend_set_framebuffer_state_no_attach(ctx->grctx, width, height, layers, samples);
+   return 0;
+}
+
 static int vrend_decode_clear(struct vrend_decode_ctx *ctx, int length)
 {
    union pipe_color_union color;
@@ -175,7 +207,7 @@ static float uif(unsigned int ui)
 static int vrend_decode_set_viewport_state(struct vrend_decode_ctx *ctx, int length)
 {
    struct pipe_viewport_state vps[PIPE_MAX_VIEWPORTS];
-   int i, v;
+   uint i, v;
    uint32_t num_viewports, start_slot;
    if (length < 1)
       return EINVAL;
@@ -277,9 +309,10 @@ static int vrend_decode_set_vertex_buffers(struct vrend_decode_ctx *ctx, uint16_
 
 static int vrend_decode_set_sampler_views(struct vrend_decode_ctx *ctx, uint16_t length)
 {
-   int num_samps;
-   int i;
-   uint32_t shader_type, start_slot;
+   uint32_t num_samps;
+   uint32_t i;
+   uint32_t shader_type;
+   uint32_t start_slot;
 
    if (length < 2)
       return EINVAL;
@@ -352,7 +385,9 @@ static int vrend_decode_draw_vbo(struct vrend_decode_ctx *ctx, int length)
 {
    struct pipe_draw_info info;
    uint32_t cso;
-   if (length != VIRGL_DRAW_VBO_SIZE)
+   uint32_t handle = 0, indirect_draw_count_handle = 0;
+   if (length != VIRGL_DRAW_VBO_SIZE && length != VIRGL_DRAW_VBO_SIZE_TESS &&
+       length != VIRGL_DRAW_VBO_SIZE_INDIRECT)
       return EINVAL;
    memset(&info, 0, sizeof(struct pipe_draw_info));
 
@@ -368,10 +403,23 @@ static int vrend_decode_draw_vbo(struct vrend_decode_ctx *ctx, int length)
    info.min_index = get_buf_entry(ctx, VIRGL_DRAW_VBO_MIN_INDEX);
    info.max_index = get_buf_entry(ctx, VIRGL_DRAW_VBO_MAX_INDEX);
 
+   if (length >= VIRGL_DRAW_VBO_SIZE_TESS) {
+      info.vertices_per_patch = get_buf_entry(ctx, VIRGL_DRAW_VBO_VERTICES_PER_PATCH);
+      info.drawid = get_buf_entry(ctx, VIRGL_DRAW_VBO_DRAWID);
+   }
+
+   if (length == VIRGL_DRAW_VBO_SIZE_INDIRECT) {
+      handle = get_buf_entry(ctx, VIRGL_DRAW_VBO_INDIRECT_HANDLE);
+      info.indirect.offset = get_buf_entry(ctx, VIRGL_DRAW_VBO_INDIRECT_OFFSET);
+      info.indirect.stride = get_buf_entry(ctx, VIRGL_DRAW_VBO_INDIRECT_STRIDE);
+      info.indirect.draw_count = get_buf_entry(ctx, VIRGL_DRAW_VBO_INDIRECT_DRAW_COUNT);
+      info.indirect.indirect_draw_count_offset = get_buf_entry(ctx, VIRGL_DRAW_VBO_INDIRECT_DRAW_COUNT_OFFSET);
+      indirect_draw_count_handle = get_buf_entry(ctx, VIRGL_DRAW_VBO_INDIRECT_DRAW_COUNT_HANDLE);
+   }
+
    cso = get_buf_entry(ctx, VIRGL_DRAW_VBO_COUNT_FROM_SO);
 
-   vrend_draw_vbo(ctx->grctx, &info, cso);
-   return 0;
+   return vrend_draw_vbo(ctx->grctx, &info, cso, handle, indirect_draw_count_handle);
 }
 
 static int vrend_decode_create_blend(struct vrend_decode_ctx *ctx, uint32_t handle, uint16_t length)
@@ -507,6 +555,7 @@ static int vrend_decode_create_rasterizer(struct vrend_decode_ctx *ctx, uint32_t
    ebit(line_last_pixel, 28);
    ebit(half_pixel_center, 29);
    ebit(bottom_edge_rule, 30);
+   ebit(force_persample_interp, 31);
    rs_state->point_size = uif(get_buf_entry(ctx, VIRGL_OBJ_RS_POINT_SIZE));
    rs_state->sprite_coord_enable = get_buf_entry(ctx, VIRGL_OBJ_RS_SPRITE_COORD_ENABLE);
    tmp = get_buf_entry(ctx, VIRGL_OBJ_RS_S3);
@@ -577,6 +626,7 @@ static int vrend_decode_create_sampler_state(struct vrend_decode_ctx *ctx, uint3
    state.mag_img_filter = (tmp >> 13) & 0x3;
    state.compare_mode = (tmp >> 15) & 0x1;
    state.compare_func = (tmp >> 16) & 0x7;
+   state.seamless_cube_map = (tmp >> 19) & 0x1;
 
    state.lod_bias = uif(get_buf_entry(ctx, VIRGL_OBJ_SAMPLER_STATE_LOD_BIAS));
    state.min_lod = uif(get_buf_entry(ctx, VIRGL_OBJ_SAMPLER_STATE_MIN_LOD));
@@ -613,8 +663,10 @@ static int vrend_decode_create_ve(struct vrend_decode_ctx *ctx, uint32_t handle,
          ve[i].instance_divisor = get_buf_entry(ctx, VIRGL_OBJ_VERTEX_ELEMENTS_V0_INSTANCE_DIVISOR(i));
          ve[i].vertex_buffer_index = get_buf_entry(ctx, VIRGL_OBJ_VERTEX_ELEMENTS_V0_VERTEX_BUFFER_INDEX(i));
 
-         if (ve[i].vertex_buffer_index >= PIPE_MAX_ATTRIBS)
+         if (ve[i].vertex_buffer_index >= PIPE_MAX_ATTRIBS) {
+            FREE(ve);
             return EINVAL;
+         }
 
          ve[i].src_format = get_buf_entry(ctx, VIRGL_OBJ_VERTEX_ELEMENTS_V0_SRC_FORMAT(i));
       }
@@ -639,7 +691,7 @@ static int vrend_decode_create_query(struct vrend_decode_ctx *ctx, uint32_t hand
 
    tmp = get_buf_entry(ctx, VIRGL_OBJ_QUERY_TYPE_INDEX);
    query_type = VIRGL_OBJ_QUERY_TYPE(tmp);
-   query_index = VIRGL_OBJ_QUERY_INDEX(tmp);
+   query_index = (tmp >> 16) & 0xffff;
 
    offset = get_buf_entry(ctx, VIRGL_OBJ_QUERY_OFFSET);
    res_handle = get_buf_entry(ctx, VIRGL_OBJ_QUERY_RES_HANDLE);
@@ -771,7 +823,8 @@ static int vrend_decode_set_scissor_state(struct vrend_decode_ctx *ctx, int leng
 {
    struct pipe_scissor_state ss[PIPE_MAX_VIEWPORTS];
    uint32_t temp;
-   uint32_t num_scissor, start_slot;
+   int32_t num_scissor;
+   uint32_t start_slot;
    int s;
    if (length < 1)
       return EINVAL;
@@ -840,6 +893,17 @@ static int vrend_decode_set_sample_mask(struct vrend_decode_ctx *ctx, int length
    return 0;
 }
 
+static int vrend_decode_set_min_samples(struct vrend_decode_ctx *ctx, int length)
+{
+   unsigned min_samples;
+
+   if (length != VIRGL_SET_MIN_SAMPLES_SIZE)
+      return EINVAL;
+   min_samples = get_buf_entry(ctx, VIRGL_SET_MIN_SAMPLES_MASK);
+   vrend_set_min_samples(ctx->grctx, min_samples);
+   return 0;
+}
+
 static int vrend_decode_resource_copy_region(struct vrend_decode_ctx *ctx, int length)
 {
    struct pipe_box box;
@@ -939,8 +1003,7 @@ static int vrend_decode_begin_query(struct vrend_decode_ctx *ctx, int length)
 
    uint32_t handle = get_buf_entry(ctx, VIRGL_QUERY_BEGIN_HANDLE);
 
-   vrend_begin_query(ctx->grctx, handle);
-   return 0;
+   return vrend_begin_query(ctx->grctx, handle);
 }
 
 static int vrend_decode_end_query(struct vrend_decode_ctx *ctx, int length)
@@ -950,8 +1013,7 @@ static int vrend_decode_end_query(struct vrend_decode_ctx *ctx, int length)
 
    uint32_t handle = get_buf_entry(ctx, VIRGL_QUERY_END_HANDLE);
 
-   vrend_end_query(ctx->grctx, handle);
-   return 0;
+   return vrend_end_query(ctx->grctx, handle);
 }
 
 static int vrend_decode_get_query_result(struct vrend_decode_ctx *ctx, int length)
@@ -1025,13 +1087,145 @@ static int vrend_decode_bind_shader(struct vrend_decode_ctx *ctx, int length)
    return 0;
 }
 
+static int vrend_decode_set_tess_state(struct vrend_decode_ctx *ctx,
+				       int length)
+{
+   float tess_factors[6];
+   int i;
+
+   if (length != VIRGL_TESS_STATE_SIZE)
+      return EINVAL;
+
+   for (i = 0; i < 6; i++) {
+      tess_factors[i] = uif(get_buf_entry(ctx, i + 1));
+   }
+   vrend_set_tess_state(ctx->grctx, tess_factors);
+   return 0;
+}
+
+static int vrend_decode_set_shader_buffers(struct vrend_decode_ctx *ctx, uint16_t length)
+{
+   int num_ssbo;
+   uint32_t shader_type, start_slot;
+
+   if (length < 2)
+      return EINVAL;
+
+   num_ssbo = (length - 2) / VIRGL_SET_SHADER_BUFFER_ELEMENT_SIZE;
+   shader_type = get_buf_entry(ctx, VIRGL_SET_SHADER_BUFFER_SHADER_TYPE);
+   start_slot = get_buf_entry(ctx, VIRGL_SET_SHADER_BUFFER_START_SLOT);
+   if (shader_type >= PIPE_SHADER_TYPES)
+      return EINVAL;
+
+   if (num_ssbo < 1)
+      return 0;
+
+   if (start_slot + num_ssbo > PIPE_MAX_SHADER_BUFFERS)
+      return EINVAL;
+
+   for (int i = 0; i < num_ssbo; i++) {
+      uint32_t offset = get_buf_entry(ctx, VIRGL_SET_SHADER_BUFFER_OFFSET(i));
+      uint32_t buf_len = get_buf_entry(ctx, VIRGL_SET_SHADER_BUFFER_LENGTH(i));
+      uint32_t handle = get_buf_entry(ctx, VIRGL_SET_SHADER_BUFFER_RES_HANDLE(i));
+      vrend_set_single_ssbo(ctx->grctx, shader_type, start_slot + i, offset, buf_len,
+                            handle);
+   }
+   return 0;
+}
+
+static int vrend_decode_set_atomic_buffers(struct vrend_decode_ctx *ctx, uint16_t length)
+{
+   int num_abo;
+   uint32_t start_slot;
+
+   if (length < 2)
+      return EINVAL;
+
+   num_abo = (length - 1) / VIRGL_SET_ATOMIC_BUFFER_ELEMENT_SIZE;
+   start_slot = get_buf_entry(ctx, VIRGL_SET_ATOMIC_BUFFER_START_SLOT);
+   if (num_abo < 1)
+      return 0;
+
+   if (start_slot + num_abo > PIPE_MAX_HW_ATOMIC_BUFFERS)
+      return EINVAL;
+
+   for (int i = 0; i < num_abo; i++) {
+      uint32_t offset = get_buf_entry(ctx, i * VIRGL_SET_ATOMIC_BUFFER_ELEMENT_SIZE + 2);
+      uint32_t buf_len = get_buf_entry(ctx, i * VIRGL_SET_ATOMIC_BUFFER_ELEMENT_SIZE + 3);
+      uint32_t handle = get_buf_entry(ctx, i * VIRGL_SET_ATOMIC_BUFFER_ELEMENT_SIZE + 4);
+      vrend_set_single_abo(ctx->grctx, start_slot + i, offset, buf_len, handle);
+   }
+
+   return 0;
+}
+
+static int vrend_decode_set_shader_images(struct vrend_decode_ctx *ctx, uint16_t length)
+{
+   int num_images;
+   uint32_t shader_type, start_slot;
+   if (length < 2)
+      return EINVAL;
+
+   num_images = (length - 2) / VIRGL_SET_SHADER_IMAGE_ELEMENT_SIZE;
+   shader_type = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_SHADER_TYPE);
+   start_slot = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_START_SLOT);
+   if (shader_type >= PIPE_SHADER_TYPES)
+      return EINVAL;
+
+   if (num_images < 1) {
+      return 0;
+   }
+   if (start_slot + num_images > PIPE_MAX_SHADER_IMAGES)
+      return EINVAL;
+
+   for (int i = 0; i < num_images; i++) {
+      uint32_t format = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_FORMAT(i));
+      uint32_t access = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_ACCESS(i));
+      uint32_t layer_offset = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_LAYER_OFFSET(i));
+      uint32_t level_size = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_LEVEL_SIZE(i));
+      uint32_t handle = get_buf_entry(ctx, VIRGL_SET_SHADER_IMAGE_RES_HANDLE(i));
+      vrend_set_single_image_view(ctx->grctx, shader_type, start_slot + i, format, access,
+                                  layer_offset, level_size, handle);
+   }
+   return 0;
+}
+
+static int vrend_decode_memory_barrier(struct vrend_decode_ctx *ctx, uint16_t length)
+{
+   if (length != VIRGL_MEMORY_BARRIER_SIZE)
+      return EINVAL;
+
+   unsigned flags = get_buf_entry(ctx, VIRGL_MEMORY_BARRIER_FLAGS);
+   vrend_memory_barrier(ctx->grctx, flags);
+   return 0;
+}
+
+static int vrend_decode_launch_grid(struct vrend_decode_ctx *ctx, uint16_t length)
+{
+   uint32_t block[3], grid[3];
+   uint32_t indirect_handle, indirect_offset;
+   if (length != VIRGL_LAUNCH_GRID_SIZE)
+      return EINVAL;
+
+   block[0] = get_buf_entry(ctx, VIRGL_LAUNCH_BLOCK_X);
+   block[1] = get_buf_entry(ctx, VIRGL_LAUNCH_BLOCK_Y);
+   block[2] = get_buf_entry(ctx, VIRGL_LAUNCH_BLOCK_Z);
+   grid[0] = get_buf_entry(ctx, VIRGL_LAUNCH_GRID_X);
+   grid[1] = get_buf_entry(ctx, VIRGL_LAUNCH_GRID_Y);
+   grid[2] = get_buf_entry(ctx, VIRGL_LAUNCH_GRID_Z);
+   indirect_handle = get_buf_entry(ctx, VIRGL_LAUNCH_INDIRECT_HANDLE);
+   indirect_offset = get_buf_entry(ctx, VIRGL_LAUNCH_INDIRECT_OFFSET);
+   vrend_launch_grid(ctx->grctx, block, grid, indirect_handle, indirect_offset);
+   return 0;
+}
+
 static int vrend_decode_set_streamout_targets(struct vrend_decode_ctx *ctx,
                                               uint16_t length)
 {
    uint32_t handles[16];
    uint32_t num_handles = length - 1;
    uint32_t append_bitmask;
-   int i;
+   uint i;
 
    if (length < 1)
       return EINVAL;
@@ -1045,6 +1239,16 @@ static int vrend_decode_set_streamout_targets(struct vrend_decode_ctx *ctx,
    return 0;
 }
 
+static int vrend_decode_texture_barrier(struct vrend_decode_ctx *ctx, uint16_t length)
+{
+   if (length != VIRGL_TEXTURE_BARRIER_SIZE)
+      return EINVAL;
+
+   unsigned flags = get_buf_entry(ctx, VIRGL_TEXTURE_BARRIER_FLAGS);
+   vrend_texture_barrier(ctx->grctx, flags);
+   return 0;
+}
+
 void vrend_renderer_context_create_internal(uint32_t handle, uint32_t nlen,
                                             const char *debug_name)
 {
@@ -1226,6 +1430,9 @@ int vrend_decode_block(uint32_t ctx_id, uint32_t *block, int ndw)
       case VIRGL_CCMD_SET_SAMPLE_MASK:
          ret = vrend_decode_set_sample_mask(gdctx, len);
          break;
+      case VIRGL_CCMD_SET_MIN_SAMPLES:
+         ret = vrend_decode_set_min_samples(gdctx, len);
+         break;
       case VIRGL_CCMD_SET_STREAMOUT_TARGETS:
          ret = vrend_decode_set_streamout_targets(gdctx, len);
          break;
@@ -1247,6 +1454,30 @@ int vrend_decode_block(uint32_t ctx_id, uint32_t *block, int ndw)
       case VIRGL_CCMD_BIND_SHADER:
          ret = vrend_decode_bind_shader(gdctx, len);
          break;
+      case VIRGL_CCMD_SET_TESS_STATE:
+         ret = vrend_decode_set_tess_state(gdctx, len);
+         break;
+      case VIRGL_CCMD_SET_SHADER_BUFFERS:
+         ret = vrend_decode_set_shader_buffers(gdctx, len);
+         break;
+      case VIRGL_CCMD_SET_SHADER_IMAGES:
+         ret = vrend_decode_set_shader_images(gdctx, len);
+         break;
+      case VIRGL_CCMD_SET_ATOMIC_BUFFERS:
+         ret = vrend_decode_set_atomic_buffers(gdctx, len);
+         break;
+      case VIRGL_CCMD_MEMORY_BARRIER:
+         ret = vrend_decode_memory_barrier(gdctx, len);
+         break;
+      case VIRGL_CCMD_LAUNCH_GRID:
+         ret = vrend_decode_launch_grid(gdctx, len);
+         break;
+      case VIRGL_CCMD_SET_FRAMEBUFFER_STATE_NO_ATTACH:
+         ret = vrend_decode_set_framebuffer_state_no_attach(gdctx, len);
+         break;
+      case VIRGL_CCMD_TEXTURE_BARRIER:
+         ret = vrend_decode_texture_barrier(gdctx, len);
+         break;
       default:
          ret = EINVAL;
       }
