@@ -30,6 +30,7 @@
 #include "virgl_protocol.h"
 #include "vrend_iov.h"
 #include "virgl_hw.h"
+#include <epoxy/gl.h>
 
 typedef void *virgl_gl_context;
 typedef void *virgl_gl_drawable;
@@ -43,6 +44,11 @@ struct virgl_gl_ctx_param {
 extern int vrend_dump_shaders;
 struct vrend_context;
 
+/* Number of mipmap levels for which to keep the backing iov offsets.
+ * Value mirrored from mesa/virgl
+ */
+#define VR_MAX_TEXTURE_2D_LEVELS 15
+
 struct vrend_resource {
    struct pipe_resource base;
    GLuint id;
@@ -54,29 +60,28 @@ struct vrend_resource {
 
    GLuint tbo_tex_id;/* tbos have two ids to track */
    bool y_0_top;
+   bool is_buffer;
 
    GLuint handle;
 
    char *ptr;
    struct iovec *iov;
    uint32_t num_iovs;
+   uint64_t mipmap_offsets[VR_MAX_TEXTURE_2D_LEVELS];
 };
 
-/* assume every format is sampler friendly */
-#define VREND_BIND_SAMPLER (1 << 0)
-#define VREND_BIND_RENDER (1 << 1)
-#define VREND_BIND_DEPTHSTENCIL (1 << 2)
+#define VIRGL_BIND_NEED_SWIZZLE (1 << 28)
+#define VIRGL_BIND_CAN_TEXTURE_STORAGE (1 << 29)
 
-#define VREND_BIND_NEED_SWIZZLE (1 << 28)
 
 struct vrend_format_table {
    enum virgl_formats format;
    GLenum internalformat;
    GLenum glformat;
    GLenum gltype;
+   uint8_t swizzle[4];
    uint32_t bindings;
    int flags;
-   uint8_t swizzle[4];
 };
 
 struct vrend_transfer_info {
@@ -105,9 +110,11 @@ int vrend_renderer_init(struct vrend_if_cbs *cbs, uint32_t flags);
 
 void vrend_insert_format(struct vrend_format_table *entry, uint32_t bindings);
 void vrend_insert_format_swizzle(int override_format, struct vrend_format_table *entry, uint32_t bindings, uint8_t swizzle[4]);
+const struct vrend_format_table *vrend_get_format_table_entry(enum virgl_formats format);
 int vrend_create_shader(struct vrend_context *ctx,
                         uint32_t handle,
                         const struct pipe_stream_output_info *stream_output,
+                        uint32_t req_local_mem,
                         const char *shd_text, uint32_t offlen, uint32_t num_tokens,
                         uint32_t type, uint32_t pkt_length);
 
@@ -122,9 +129,9 @@ void vrend_clear(struct vrend_context *ctx,
                  const union pipe_color_union *color,
                  double depth, unsigned stencil);
 
-void vrend_draw_vbo(struct vrend_context *ctx,
-                    const struct pipe_draw_info *info,
-                    uint32_t cso);
+int vrend_draw_vbo(struct vrend_context *ctx,
+                   const struct pipe_draw_info *info,
+                   uint32_t cso, uint32_t indirect_handle, uint32_t indirect_draw_count_handle);
 
 void vrend_set_framebuffer_state(struct vrend_context *ctx,
                                  uint32_t nr_cbufs, uint32_t surf_handle[PIPE_MAX_COLOR_BUFS],
@@ -136,18 +143,6 @@ int vrend_renderer_context_create(uint32_t handle, uint32_t nlen, const char *na
 void vrend_renderer_context_create_internal(uint32_t handle, uint32_t nlen, const char *name);
 void vrend_renderer_context_destroy(uint32_t handle);
 
-/* virgl bind flags - these are compatible with mesa 10.5 gallium.
-   but are fixed, no other should be passed to virgl either. */
-#define VREND_RES_BIND_DEPTH_STENCIL (1 << 0)
-#define VREND_RES_BIND_RENDER_TARGET (1 << 1)
-#define VREND_RES_BIND_SAMPLER_VIEW  (1 << 3)
-#define VREND_RES_BIND_VERTEX_BUFFER (1 << 4)
-#define VREND_RES_BIND_INDEX_BUFFER  (1 << 5)
-#define VREND_RES_BIND_CONSTANT_BUFFER (1 << 6)
-#define VREND_RES_BIND_STREAM_OUTPUT (1 << 11)
-#define VREND_RES_BIND_CURSOR        (1 << 16)
-#define VREND_RES_BIND_CUSTOM        (1 << 17)
-
 struct vrend_renderer_resource_create_args {
    uint32_t handle;
    enum pipe_texture_target target;
@@ -162,7 +157,7 @@ struct vrend_renderer_resource_create_args {
    uint32_t flags;
 };
 
-int vrend_renderer_resource_create(struct vrend_renderer_resource_create_args *args, struct iovec *iov, uint32_t num_iovs);
+int vrend_renderer_resource_create(struct vrend_renderer_resource_create_args *args, struct iovec *iov, uint32_t num_iovs, void *image_eos);
 
 void vrend_renderer_resource_unref(uint32_t handle);
 
@@ -218,7 +213,7 @@ void vrend_set_num_sampler_views(struct vrend_context *ctx,
                                  int num_sampler_views);
 void vrend_set_single_sampler_view(struct vrend_context *ctx,
                                    uint32_t shader_type,
-                                   int index,
+                                   uint32_t index,
                                    uint32_t res_handle);
 
 void vrend_object_bind_blend(struct vrend_context *ctx,
@@ -237,7 +232,33 @@ void vrend_set_index_buffer(struct vrend_context *ctx,
                             uint32_t res_handle,
                             uint32_t index_size,
                             uint32_t offset);
-
+void vrend_set_single_image_view(struct vrend_context *ctx,
+                                 uint32_t shader_type,
+                                 int index,
+                                 uint32_t format, uint32_t access,
+                                 uint32_t layer_offset, uint32_t level_size,
+                                 uint32_t handle);
+void vrend_set_single_ssbo(struct vrend_context *ctx,
+                           uint32_t shader_type,
+                           int index,
+                           uint32_t offset, uint32_t length,
+                           uint32_t handle);
+void vrend_set_single_abo(struct vrend_context *ctx,
+                          int index,
+                          uint32_t offset, uint32_t length,
+                          uint32_t handle);
+void vrend_memory_barrier(struct vrend_context *ctx,
+                          unsigned flags);
+void vrend_launch_grid(struct vrend_context *ctx,
+                       uint32_t *block,
+                       uint32_t *grid,
+                       uint32_t indirect_handle,
+                       uint32_t indirect_offset);
+void vrend_set_framebuffer_state_no_attach(struct vrend_context *ctx,
+                                           uint32_t width, uint32_t height,
+                                           uint32_t layers, uint32_t samples);
+void vrend_texture_barrier(struct vrend_context *ctx,
+                           unsigned flags);
 #define VREND_TRANSFER_WRITE 1
 #define VREND_TRANSFER_READ 2
 int vrend_renderer_transfer_iov(const struct vrend_transfer_info *info, int transfer_mode);
@@ -263,6 +284,7 @@ void vrend_set_polygon_stipple(struct vrend_context *ctx, struct pipe_poly_stipp
 
 void vrend_set_clip_state(struct vrend_context *ctx, struct pipe_clip_state *ucp);
 void vrend_set_sample_mask(struct vrend_context *ctx, unsigned sample_mask);
+void vrend_set_min_samples(struct vrend_context *ctx, unsigned min_samples);
 
 void vrend_set_constants(struct vrend_context *ctx,
                          uint32_t shader,
@@ -274,6 +296,8 @@ void vrend_set_uniform_buffer(struct vrend_context *ctx, uint32_t shader,
                               uint32_t index, uint32_t offset, uint32_t length,
                               uint32_t res_handle);
 
+void vrend_set_tess_state(struct vrend_context *ctx, const float tess_factors[6]);
+
 void vrend_renderer_fini(void);
 
 int vrend_decode_block(uint32_t ctx_id, uint32_t *block, int ndw);
@@ -293,8 +317,8 @@ int vrend_create_query(struct vrend_context *ctx, uint32_t handle,
                        uint32_t query_type, uint32_t query_index,
                        uint32_t res_handle, uint32_t offset);
 
-void vrend_begin_query(struct vrend_context *ctx, uint32_t handle);
-void vrend_end_query(struct vrend_context *ctx, uint32_t handle);
+int vrend_begin_query(struct vrend_context *ctx, uint32_t handle);
+int vrend_end_query(struct vrend_context *ctx, uint32_t handle);
 void vrend_get_query_result(struct vrend_context *ctx, uint32_t handle,
                             uint32_t wait);
 void vrend_render_condition(struct vrend_context *ctx,
@@ -311,7 +335,10 @@ void vrend_renderer_fill_caps(uint32_t set, uint32_t version,
 
 GLint64 vrend_renderer_get_timestamp(void);
 
-void vrend_build_format_list(void);
+void vrend_build_format_list_common(void);
+void vrend_build_format_list_gl(void);
+void vrend_build_format_list_gles(void);
+void vrend_check_texture_storage(struct vrend_format_table *table);
 
 int vrend_renderer_resource_attach_iov(int res_handle, struct iovec *iov,
                                        int num_iovs);
@@ -352,6 +379,7 @@ int vrend_renderer_resource_get_info(int res_handle,
                                      struct vrend_renderer_resource_info *info);
 
 #define VREND_CAP_SET 1
+#define VREND_CAP_SET2 2
 
 void vrend_renderer_get_cap_set(uint32_t cap_set, uint32_t *max_ver,
                                 uint32_t *max_size);
@@ -366,17 +394,31 @@ void vrend_fb_bind_texture(struct vrend_resource *res,
                            uint32_t level, uint32_t layer);
 bool vrend_is_ds_format(enum virgl_formats format);
 bool vrend_format_is_emulated_alpha(enum virgl_formats format);
+boolean format_is_copy_compatible(enum pipe_format src, enum pipe_format dst,
+                                  boolean allow_compressed);
+
 /* blitter interface */
 void vrend_renderer_blit_gl(struct vrend_context *ctx,
                             struct vrend_resource *src_res,
                             struct vrend_resource *dst_res,
-                            const struct pipe_blit_info *info);
+                            const struct pipe_blit_info *info,
+                            bool has_texture_srgb_decode);
+void vrend_blitter_fini(void);
 
 void vrend_renderer_reset(void);
 int vrend_renderer_get_poll_fd(void);
 void vrend_decode_reset(bool ctx_0_only);
-#define VREND_GL_VER_MAJOR 3
-#define VREND_GL_VER_MINOR 1
+
+unsigned vrend_renderer_query_multisample_caps(unsigned max_samples,
+                                               struct virgl_caps_v2 *caps);
+
+struct gl_version {
+   uint32_t major;
+   uint32_t minor;
+};
+
+static const struct gl_version gl_versions[] = { {4,5}, {4,4}, {4,3}, {4,2}, {4,1}, {4,0},
+                                                 {3,3}, {3,2}, {3,1}, {3,0} };
 
 extern struct vrend_if_cbs *vrend_clicbs;
 #endif
